from Products.CMFCore.ActionInformation import Action
from Products.CMFCore.utils import getToolByName


def setupVarious(context):
    """ Only in plonetheme.sunburst profile """

    if context.readDataFile('plonetheme.bootstrap_various.txt') is None:
        return

    # Do what plonetheme.sunburst does
    # Copy the plone_setup action to the user category
    atool = getToolByName(context.getSite(), 'portal_actions')
    user_category = atool.user
    if 'plone_setup' in user_category:
        return

    state = dict(atool.site_actions.plone_setup.propertyItems())
    state['visible'] = True
    new_action = Action('plone_setup', **state)
    user_category['plone_setup'] = new_action

    position = user_category.getObjectPosition('preferences') + 1
    user_category.moveObjectToPosition('plone_setup', position)



from zope.interface import Interface
from plone.theme.interfaces import IDefaultPloneLayer

class IThemeSpecific(IDefaultPloneLayer):
    """Marker interface that defines a Zope 3 browser layer.
    """


class IBootstrapUtils(Interface):
    def bootstrap():
        pass


class IBootstrapView(Interface):
    """ """

    def getColumnsClasses(view=None):
        """ A helper method to return the clases for the columns of the site
            it should return a dict with three elements:'one', 'two', 'content'
            Each of them should contain the classnames for the first (leftmost)
            second (rightmost) and middle column
        """

    def getViewportValues(view=None):
        """ Determine the value of the viewport meta-tag """


from Products.CMFCore.utils import getToolByName

import logging

PROFILE_ID = 'profile-plonetheme.bootstrap:default'


def common(context, logger=None):
    if logger is None:
        # Called as upgrade step: define our own logger.
        logger = logging.getLogger('plonetheme.bootstrap')

    setup = getToolByName(context, 'portal_setup')
    setup.runAllImportStepsFromProfile(PROFILE_ID)

    logger.info("Migrated from previous version")


def remove_old_2_css(context, logger=None):
    if logger is None:
        # Called as upgrade step: define our own logger.
        logger = logging.getLogger('plonetheme.bootstrap')

    csstool = getToolByName(context, 'portal_css')
    csstool.unregisterResource('++resource++collective.js.bootstrap/css/bootstrap-responsive.min.css')

    logger.info("Migrated from previous version")



# -*- coding: utf-8 -*-
import os

from plone.app.testing.interfaces import PLONE_SITE_ID
from plone.testing import z2


PORT = os.environ.get('ZSERVER_PORT', z2.ZServer.port)
PORT = os.environ.get('PLONE_TESTING_PORT', PORT)
ZSERVER_PORT = PORT
SELENIUM_IMPLICIT_WAIT = os.environ.get('SELENIUM_IMPLICIT_WAIT', '0.1s')
SELENIUM_TIMEOUT = os.environ.get('SELENIUM_IMPLICIT_WAIT', '20s')

ZOPE_HOST = os.environ.get('ZOPE_HOST', "localhost")
ZOPE_URL = os.environ.get('ZOPE_URL', "http://%s:%s" % (ZOPE_HOST, PORT))
PLONE_SITE_ID = os.environ.get('PLONE_SITE_ID', PLONE_SITE_ID)
PLONE_URL = os.environ.get('PLONE_URL', "%s/%s" % (ZOPE_URL, PLONE_SITE_ID))
BROWSER = os.environ.get('BROWSER', "Firefox")
REMOTE_URL = os.environ.get('REMOTE_URL', "")
DESIRED_CAPABILITIES = os.environ.get('DESIRED_CAPABILITIES', "")

TEST_FOLDER = os.environ.get('TEST_FOLDER', "%s/robot-test-folder" % PLONE_URL)

# Imports to fill in variables
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.testing import TEST_USER_NAME
from plone.app.testing import TEST_USER_PASSWORD
from plone.app.testing import TEST_USER_ID



import unittest2 as unittest
import os
import robotsuite
from plone.testing import layered
from plonetheme.bootstrap.testing import BOOTSTRAPTHEME_ROBOT


def test_suite():
    suite = unittest.TestSuite()
    current_dir = os.path.abspath(os.path.dirname(__file__))
    robot_dir = os.path.join(current_dir, 'robot')
    robot_tests = [
        os.path.join('robot', doc) for doc in
        os.listdir(robot_dir) if doc.endswith('.robot') and
        doc.startswith('test_')
    ]
    for test in robot_tests:
        suite.addTests([
            layered(robotsuite.RobotTestSuite(test),
                    layer=BOOTSTRAPTHEME_ROBOT),
        ])
    return suite



# -*- coding: utf-8 -*-
from plone.app.robotframework.remote import RemoteLibrary


class CheckErrorLog(RemoteLibrary):

    def error_log_is_empty(self, *args):
        """
        Ensure the site error_log has no entries in it
        """
        entries = self.aq_parent.aq_base.error_log.getLogEntries()
        if entries:
            error = u"Expected zero entries in site error log, found %i instead: %s"
            raise Exception(error % len(entries), repr(entries))



from Acquisition import aq_inner
from plone.app.layout.viewlets.common import ViewletBase
from zope.component import getMultiAdapter
from Products.CMFCore.utils import getToolByName

class BootstrapDropdownMenu(ViewletBase):

    def navtree(self):
        context = aq_inner(self.context)
        view = getMultiAdapter((context, self.request),
                               name='sitemap_builder_view')
        data = view.siteMap()
        bottomLevel = 5
        portal_tabs_view = getMultiAdapter((context, self.request),
                                           name='portal_tabs_view')
        portal_tabs = portal_tabs_view.topLevelTabs()
        selected_tabs = self.selectedTabs(portal_tabs=portal_tabs)
        selected_portal_tab = selected_tabs['portal']
        # XXX: The recursion should probably be done in python code

        return context.homepage_sections(
            children=data.get('children', []),
            level=1,
            bottomLevel=bottomLevel,
            selected_portal_tab=selected_portal_tab,
        )


    def selectedTabs(self, default_tab='index_html', portal_tabs=()):
        plone_url = getToolByName(self.context, 'portal_url')()
        plone_url_len = len(plone_url)
        request = self.request
        valid_actions = []

        url = request['URL']
        path = url[plone_url_len:]

        for action in portal_tabs:
            if not action['url'].startswith(plone_url):
                # In this case the action url is an external link. Then, we
                # avoid issues (bad portal_tab selection) continuing with next
                # action.
                continue
            action_path = action['url'][plone_url_len:]
            if not action_path.startswith('/'):
                action_path = '/' + action_path
            if path.startswith(action_path + '/') or path == action_path:
                # Make a list of the action ids, along with the path length
                # for choosing the longest (most relevant) path.
                valid_actions.append((len(action_path), action['id']))

        # Sort by path length, the longest matching path wins
        valid_actions.sort()
        if valid_actions:
            return {'portal' : valid_actions[-1][1]}

        return {'portal' : default_tab}


from zope.interface import Interface, implements
from zope.component import getMultiAdapter

from Products.Five.browser import BrowserView
from Acquisition import aq_inner, aq_parent

from plone.memoize.view import memoize
from plone.app.layout.navigation.interfaces import INavigationRoot
from plone.app.layout.navigation.root import getNavigationRootObject

class INavigationRootView(Interface):
    def is_navigation_root_or_default_page():
        """ return whether the current object is the navigation
            root element or the default page of a navigation root
            item. This is similar to @@ploneview/isPortalOrPortalDefaultPage method
        """

    def navigation_root_title():
        """ return the title of the (upwards) nearest INavigationRoot
            enabled item
        """

class NavigationRoot(BrowserView):
    implements(INavigationRootView)

    @memoize
    def is_navigation_root_or_default_page(self):
        context = aq_inner(self.context)
        context_state = getMultiAdapter(
            (context, self.request), name=u'plone_context_state')
        if context_state.is_default_page():
            return INavigationRoot.providedBy(aq_parent(context))
        else:
            return INavigationRoot.providedBy(context)

    @memoize
    def navigation_root_title(self):
        context = aq_inner(self.context)
        portal = getMultiAdapter((context, self.request), name=u'plone_portal_state').portal
        navigation_root = getNavigationRootObject(context, portal)
        return navigation_root.Title()

    @memoize
    def navigation_root_url(self):
        context = aq_inner(self.context)
        portal = getMultiAdapter((context, self.request), name=u'plone_portal_state').portal
        navigation_root = getNavigationRootObject(context, portal)
        return navigation_root.absolute_url()



# -*- coding: utf-8 -*-

from Products.CMFPlone.utils import getToolByName


def isPlone42(portal):
    migration_tool = getToolByName(portal, 'portal_migration')
    core_versions = migration_tool.coreVersions()
    plone = core_versions.get('Plone', '')
    if plone:
        versions = plone.split('.')
        return versions[0] == '4' and versions[1] == '2'

    return False


def isPlone41(portal):
    migration_tool = getToolByName(portal, 'portal_migration')
    core_versions = migration_tool.coreVersions()
    plone = core_versions.get('Plone', '')
    if plone:
        versions = plone.split('.')
        return versions[0] == '4' and versions[1] == '1'

    return False


def uninstall(portal):
    """ Apply the correct so called uninstall-profile when uninstalling
        according to the installed Plone version
    """

    if isPlone42(portal):
        profile = 'profile-plonetheme.bootstrap:uninstall'
    elif isPlone41(portal):
        profile = 'profile-plonetheme.bootstrap:uninstall-plone41'
    else:
        profile = ''

    if profile:
        setup_tool = getToolByName(portal, 'portal_setup')
        setup_tool.runAllImportStepsFromProfile(profile)
        return "Ran all uninstall steps."

    else:
        return "Nothing to uninstall"



from plone.app.testing import PloneSandboxLayer
from plone.app.testing import PLONE_FIXTURE
from plone.app.testing import FunctionalTesting

from plone.testing import z2
from plone.app.robotframework.testing import AUTOLOGIN_LIBRARY_FIXTURE
from plone.app.robotframework import RemoteLibraryLayer, AutoLogin
from plonetheme.bootstrap.tests.check_error_log import CheckErrorLog


class BootstrapTheme(PloneSandboxLayer):

    defaultBases = (PLONE_FIXTURE, )

    def setUpZope(self, app, configurationContext):
        # Load ZCML
        import plonetheme.bootstrap
        self.loadZCML(package=plonetheme.bootstrap)

    def setUpPloneSite(self, portal):
        # Install into Plone site using portal_quickinstaller
        portal.portal_quickinstaller.installProduct('plonetheme.bootstrap')
        # A different install profile will be chosen for Plone>4.1

        portal.portal_css.cookResources()
        portal.portal_javascripts.cookResources()


BOOTSTRAPTHEME_FIXTURE = BootstrapTheme()

CHECK_ERROR_LOG_FIXTURE = RemoteLibraryLayer(
    bases=(PLONE_FIXTURE,),
    libraries=(CheckErrorLog, AutoLogin),
    name="CheckErrorLogRemoteLibrary:RobotRemote"
)

BOOTSTRAPTHEME_ROBOT = FunctionalTesting(
    bases=(CHECK_ERROR_LOG_FIXTURE, BOOTSTRAPTHEME_FIXTURE, z2.ZSERVER_FIXTURE),
    name='BootstrapTheme:Robot')



from setuptools import setup, find_packages
import os

version = '2.0'

setup(name='plonetheme.bootstrap',
      version=version,
      description="bootstrap css integration",
      long_description=open("README.rst").read() + "\n" +
                       open(os.path.join("docs", "HISTORY.txt")).read(),
      # Get more strings from
      # http://pypi.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
          "Framework :: Plone",
          "Programming Language :: Python",
      ],
      keywords='plone theme twitterbootstrap',
      author='Nathan van Gheem, Mikel Larreategi',
      author_email='mlarreategi@codesyntax.com',
      url='https://github.com/collective/plonetheme.bootstrap',
      license='GPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['plonetheme'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          'z3c.jbot',
          'plone.browserlayer',
          'collective.js.bootstrap>=3.0',
          # -*- Extra requirements: -*-
      ],
      extras_require={
          'test': [
              'plone.app.testing[robot]>=4.2.2',
              'plone.app.robotframework',
          ]
      },
      entry_points="""
      # -*- Entry points: -*-

      [z3c.autoinclude.plugin]
      target = plone
      """
      )



from interfaces import IBootstrapView
from Products.Five import BrowserView
from zope.component import getMultiAdapter
from zope.interface import implements


class BootstrapView(BrowserView):
    implements(IBootstrapView)

    def getViewportValues(self, view=None):
        """ Determine the value of the viewport meta-tag """
        values = {'width': 'device-width',
                 'initial-scale': '1.0',
                }

        return ','.join('%s=%s' % (k, v) for k, v in values.items())

    def getColumnsClasses(self, view=None):
        """ Determine whether a column should be shown. The left column is
            called plone.leftcolumn; the right column is called
            plone.rightcolumn.
        """

        plone_view = getMultiAdapter(
            (self.context, self.request), name=u'plone')
        portal_state = getMultiAdapter(
            (self.context, self.request), name=u'plone_portal_state')

        sl = plone_view.have_portlets('plone.leftcolumn', view=view)
        sr = plone_view.have_portlets('plone.rightcolumn', view=view)

        isRTL = portal_state.is_rtl()

        # pre-fill dictionary
        columns = dict(one="", content="", two="")

        if not sl and not sr:
            # we don't have columns, thus conten takes the whole width
            columns['content'] = "col-md-12"

        elif sl and sr:
            # In case we have both columns, content takes 50% of the whole
            # width and the rest 50% is spread between the columns
            columns['one'] = "col-md-3"
            columns['content'] = "col-md-6"
            columns['two'] = "col-md-3"

        elif (sr and not sl) and not isRTL:
            # We have right column and we are NOT in RTL language
            columns['content'] = "col-md-9"
            columns['two'] = "col-md-3"

        elif (sl and not sr) and isRTL:
            # We have left column and we are in RTL language
            columns['one'] = "col-md-3"
            columns['content'] = "col-md-9"

        elif (sl and not sr) and not isRTL:
            # We have left column and we are in NOT RTL language
            columns['one'] = "col-md-3"
            columns['content'] = "col-md-9"

        # # append cell to each css-string
        # for key, value in columns.items():
        #     columns[key] = "cell " + value

        return columns
